/***************************************************/
/* Game Excelleration Kit vX                       */
/* Copyright (C) 2001 by HourGlass Software NonInc */
/***************************************************/
#ifndef __GEKPlane__
#define __GEKPlane__
/*! 
	\author MoJo
    \file GEKPlane.h
    \brief Plane Class
*/

//! Generates a normal from 3 planar points
GEKVECTOR3 GEKNormalize(const GEKVECTOR3 &kVector1, const GEKVECTOR3 &kVector2, const GEKVECTOR3 &kVector3);

//! Plane Class
struct GEKPLANE
{
public:
	GEKVECTOR3 n;
	float d;

public:
	inline GEKPLANE(void);
	inline GEKPLANE(const GEKPLANE &kPlane);
	inline GEKPLANE(float fA, float fB, float fC, float fD);
	inline GEKPLANE(const GEKVECTOR3 &kNormal, float fDistance);
	GEKPLANE(const GEKVECTOR3 &kVector1, const GEKVECTOR3 &kVector2, const GEKVECTOR3 &kVector3);

	//! Assignment Operator
	inline GEKPLANE operator = (const GEKPLANE &kPlane);

	//! Returns the distance from a plane to a point
	float Distance(const GEKVECTOR3 &kPoint) const;

	/*!
		\brief Returns the if a ray and a plane intersect
		\param kRay [in] the ray to check against
	*/
	bool Intersect(const GEKRAY &kRay) const;

	/*!
		\brief Returns the point of intersection of a ray to a plane
		\param kRay [in] the ray to check against
	*/
	GEKVECTOR3 Intersection(const GEKRAY &kRay) const;
};

#endif