#include "..\GEKXMath.h"

GEKQUATERNION GEKQUATERNION::operator * (const GEKQUATERNION &kQuat)
{
	GEKQUATERNION kResult;
	kResult.x = ((w * kQuat.x) + (x * kQuat.w) + (y * kQuat.z) - (z * kQuat.y));
	kResult.y = ((w * kQuat.y) - (x * kQuat.z) + (y * kQuat.w) + (z * kQuat.x));
	kResult.z = ((w * kQuat.z) + (x * kQuat.y) - (y * kQuat.x) + (z * kQuat.w));
	kResult.w = ((w * kQuat.w) - (x * kQuat.x) - (y * kQuat.y) - (z * kQuat.z));
	return kResult;
}

void GEKQUATERNION::operator *= (const GEKQUATERNION &kQuat)
{
	GEKQUATERNION kResult;
	kResult.x = ((w * kQuat.x) + (x * kQuat.w) + (y * kQuat.z) - (z * kQuat.y));
	kResult.y = ((w * kQuat.y) - (x * kQuat.z) + (y * kQuat.w) + (z * kQuat.x));
	kResult.z = ((w * kQuat.z) + (x * kQuat.y) - (y * kQuat.x) + (z * kQuat.w));
	kResult.w = ((w * kQuat.w) - (x * kQuat.x) - (y * kQuat.y) - (z * kQuat.z));
	*this = kResult;
}

void GEKQUATERNION::SetEuler(const GEKVECTOR3 &kEuler)
{
    float fSINx = SIN(kEuler.x * 0.5f);
    float fSINy = SIN(kEuler.y * 0.5f);
    float fSINz = SIN(kEuler.z * 0.5f);
    float fCOSx = COS(kEuler.x * 0.5f);
    float fCOSy = COS(kEuler.y * 0.5f);
    float fCOSz = COS(kEuler.z * 0.5f);
	float fSINyCOSz = (fSINy * fCOSz);
	float fSINySINz = (fSINy * fSINz);
	float fCOSyCOSz = (fCOSy * fCOSz);
	float fCOSySINz = (fCOSy * fSINz);

	x = ((fSINx * fCOSyCOSz) - (fCOSx * fSINySINz));
	y = ((fSINx * fCOSySINz) + (fCOSx * fSINyCOSz));
	z = ((fCOSx * fCOSySINz) - (fSINx * fSINyCOSz));
	w = ((fCOSx * fCOSyCOSz) + (fSINx * fSINySINz));
}

void GEKQUATERNION::SetRotation(const GEKVECTOR3 &kAxis, float fAngle)
{
	GEKVECTOR3 kNormal(((GEKVECTOR3 &)kAxis).Normal());
	float fSinA = SIN(fAngle * 0.5f);

	x = (kNormal.x * fSinA);
	y = (kNormal.y * fSinA);
	z = (kNormal.z * fSinA);
	w = COS(fAngle * 0.5f);
}

void GEKQUATERNION::SetMatrix(const GEKMATRIX &kMatrix)
{
static int aiNext[3] = {1, 2, 0};
	float fDiagonal = (kMatrix._11 + kMatrix._22 + kMatrix._33);
	if(fDiagonal > 0.0f)
	{
		float fSQRT = SQRT(fDiagonal + 1.0f);
		w = (fSQRT / 2.0f);

		fSQRT = (0.5f / fSQRT);
		x = ((kMatrix._23 - kMatrix._32) * fSQRT);
		y = ((kMatrix._31 - kMatrix._13) * fSQRT);
		z = ((kMatrix._12 - kMatrix._21) * fSQRT);
	}
	else
	{
		int iCount = 0; 
		if(kMatrix._22 > kMatrix._11) iCount = 1;
		if(kMatrix._33 > kMatrix.m[iCount][iCount]) iCount = 2;

		float afTemp[4];
		int iTemp1 = aiNext[iCount];
		int iTemp2 = aiNext[iTemp1];

		float fSQRT = SQRT((kMatrix.m[iCount][iCount] - 
						   (kMatrix.m[iTemp1][iTemp1] + 
						    kMatrix.m[iTemp2][iTemp2])) + 1.0f);
		afTemp[iCount] = (fSQRT * 0.5f);

		if(fSQRT != 0.0) fSQRT = (0.5f / fSQRT);
		afTemp[3] =      ((kMatrix.m[iTemp1][iTemp2] - kMatrix.m[iTemp2][iTemp1]) * fSQRT);
		afTemp[iTemp1] = ((kMatrix.m[iCount][iTemp1] + kMatrix.m[iTemp1][iCount]) * fSQRT);
		afTemp[iTemp2] = ((kMatrix.m[iCount][iTemp2] + kMatrix.m[iTemp2][iCount]) * fSQRT);
		x = afTemp[0];
		y = afTemp[1];
		z = afTemp[2];
		w = afTemp[3];
	}
}

GEKQUATERNION GEKQUATERNION::Normal(void)
{
	float fLength = Length();
	return GEKQUATERNION((x / fLength),
						 (y / fLength),
						 (z / fLength),
						 (w / fLength));
}

void GEKQUATERNION::Normalize(void)
{
	*this = Normal();
}

#include <math.h>
GEKQUATERNION GEKQUATERNION::Slerp(const GEKQUATERNION &kQuat, float fFactor)
{
	float fDot = Dot(kQuat);
	float fSqrt = SQRT(ABS(1.0f - fDot * fDot));
	if(ABS(fSqrt) < (_EPSILON * 100.0f)) return *this;
	if(fDot < 0.0f) fDot = -fDot;

	float fAngle = 0.0f;
	if(fDot != 0.0f) fAngle = (float)atanf(fSqrt / fDot);
	fSqrt = 1 / fSqrt;
	
	GEKQUATERNION kResult;
	kResult.x = ((x * fSqrt * SIN(fAngle * (1.0f - fFactor))) + (kQuat.x * fSqrt * SIN(fAngle * fFactor)));
	kResult.y = ((y * fSqrt * SIN(fAngle * (1.0f - fFactor))) + (kQuat.y * fSqrt * SIN(fAngle * fFactor)));
	kResult.z = ((z * fSqrt * SIN(fAngle * (1.0f - fFactor))) + (kQuat.z * fSqrt * SIN(fAngle * fFactor)));
	kResult.w = ((w * fSqrt * SIN(fAngle * (1.0f - fFactor))) + (kQuat.w * fSqrt * SIN(fAngle * fFactor)));
	return kResult;
}

GEKQUATERNION GEKQUATERNION::Lerp(const GEKQUATERNION &kQuat, float fFactor)
{
	GEKQUATERNION kResult;
	kResult.x = (x + ((kQuat.x - x) * fFactor));
	kResult.y = (y + ((kQuat.y - y) * fFactor));
	kResult.z = (z + ((kQuat.z - z) * fFactor));
	kResult.w = (w + ((kQuat.w - w) * fFactor));
	return kResult;
}